@@ -16,6 +16,7 @@ from __future__ import annotations
 from xknx.exceptions import ConversionError, CouldNotParseKNXIP, UnsupportedCEMIMessage
 from xknx.telegram import GroupAddress, IndividualAddress, Telegram
 from xknx.telegram.apci import APCI
+from xknx.telegram.tpci import TPCI, TDataGroup
 
 from .knxip_enum import CEMIFlags, CEMIMessageCode
 
@@ -29,7 +30,7 @@ class CEMIFrame:
         flags: int = 0,
         src_addr: IndividualAddress = IndividualAddress(None),
         dst_addr: GroupAddress | IndividualAddress = GroupAddress(None),
-        mpdu_len: int = 0,
+        tpci: TPCI = TDataGroup(),
         payload: APCI | None = None,
     ):
         """Initialize CEMIFrame object."""
@@ -37,7 +38,7 @@ class CEMIFrame:
         self.flags = flags
         self.src_addr = src_addr
         self.dst_addr = dst_addr
-        self.mpdu_len = mpdu_len
+        self.tpci = tpci
         self.payload = payload
 
     @staticmethod
@@ -48,7 +49,7 @@ class CEMIFrame:
     ) -> CEMIFrame:
         """Return CEMIFrame from a Telegram."""
         cemi = CEMIFrame(code=code, src_addr=src_addr)
-        # dst_addr, payload and cmd are set by telegram.setter - mpdu_len not needed for outgoing telegram
+        # dst_addr, payload and cmd are set by telegram.setter
         cemi.telegram = telegram
         return cemi
 
@@ -60,6 +61,7 @@ class CEMIFrame:
             destination_address=self.dst_addr,
             payload=self.payload,
             source_address=self.src_addr,
+            tpci=self.tpci,
         )
 
     @telegram.setter
@@ -71,20 +73,26 @@ class CEMIFrame:
             CEMIFlags.FRAME_TYPE_STANDARD
             | CEMIFlags.DO_NOT_REPEAT
             | CEMIFlags.BROADCAST
-            | CEMIFlags.PRIORITY_LOW
-            | CEMIFlags.NO_ACK_REQUESTED
+            | (
+                CEMIFlags.ACK_REQUESTED
+                if telegram.tpci.ack_request
+                else CEMIFlags.NO_ACK_REQUESTED
+            )
             | CEMIFlags.CONFIRM_NO_ERROR
             | CEMIFlags.HOP_COUNT_1ST
         )
 
         if isinstance(telegram.destination_address, GroupAddress):
-            self.flags |= CEMIFlags.DESTINATION_GROUP_ADDRESS
+            self.flags |= CEMIFlags.DESTINATION_GROUP_ADDRESS | CEMIFlags.PRIORITY_LOW
         elif isinstance(telegram.destination_address, IndividualAddress):
-            self.flags |= CEMIFlags.DESTINATION_INDIVIDUAL_ADDRESS
+            self.flags |= (
+                CEMIFlags.DESTINATION_INDIVIDUAL_ADDRESS | CEMIFlags.PRIORITY_SYSTEM
+            )
         else:
             raise TypeError()
 
         self.dst_addr = telegram.destination_address
+        self.tpci = telegram.tpci
         self.payload = telegram.payload
 
     def set_hops(self, hops: int) -> None:
@@ -96,9 +104,11 @@ class CEMIFrame:
 
     def calculated_length(self) -> int:
         """Get length of KNX/IP body."""
-        if not isinstance(self.payload, APCI):
-            raise TypeError()
-        return 10 + self.payload.calculated_length()
+        if not self.tpci.control and self.payload is not None:
+            return 10 + self.payload.calculated_length()
+        if self.tpci.control and self.payload is None:
+            return 10
+        raise TypeError("Data TPDU must have a payload; control TPDU must not.")
 
     def from_knx(self, raw: bytes) -> int:
         """Parse/deserialize from KNX/IP raw data."""
@@ -122,9 +132,7 @@ class CEMIFrame:
 
     def from_knx_data_link_layer(self, cemi: bytes) -> int:
         """Parse L_DATA_IND, CEMIMessageCode.L_DATA_REQ, CEMIMessageCode.L_DATA_CON."""
-        if len(cemi) < 11:
-            # eg. ETS Line-Scan issues L_DATA_IND with length 10 or checking for individual addresses
-            # at start of ETS programming procedure
+        if len(cemi) < 10:
             raise UnsupportedCEMIMessage(
                 f"CEMI too small. Length: {len(cemi)}; CEMI: {cemi.hex()}"
             )
@@ -138,43 +146,70 @@ class CEMIFrame:
 
         self.src_addr = IndividualAddress((cemi[4 + addil], cemi[5 + addil]))
 
-        if self.flags & CEMIFlags.DESTINATION_GROUP_ADDRESS:
-            self.dst_addr = GroupAddress((cemi[6 + addil], cemi[7 + addil]))
-        else:
-            self.dst_addr = IndividualAddress((cemi[6 + addil], cemi[7 + addil]))
+        dst_is_group_address = bool(self.flags & CEMIFlags.DESTINATION_GROUP_ADDRESS)
+        dst_raw_address = (cemi[6 + addil], cemi[7 + addil])
+        self.dst_addr = (
+            GroupAddress(dst_raw_address)
+            if dst_is_group_address
+            else IndividualAddress(dst_raw_address)
+        )
 
-        self.mpdu_len = cemi[8 + addil]
-
-        # TPCI (transport layer control information)   -> First 14 bit
-        # APCI (application layer control information) -> Last  10 bit
+        npdu_len = cemi[8 + addil]
 
         apdu = cemi[9 + addil :]
-        if len(apdu) != (self.mpdu_len + 1):
+        if len(apdu) != (npdu_len + 1):  # TCPI octet not included in NPDU length
             raise CouldNotParseKNXIP(
-                f"APDU LEN should be {self.mpdu_len} but is {len(apdu)} in CEMI: {cemi.hex()}"
+                f"APDU LEN should be {npdu_len} but is {len(apdu) - 1} in CEMI: {cemi.hex()}"
             )
 
-        tpci_apci = (apdu[0] << 8) + apdu[1]
-
+        # TPCI (transport layer control information)
+        # - with control bit set -> 8 bit; no APDU
+        # - no control bit set (data) -> First 6 bit
+        # APCI (application layer control information) -> Last  10 bit of TPCI/APCI
         try:
-            self.payload = APCI.resolve_apci(tpci_apci & 0x03FF)
-        except ConversionError:
-            raise UnsupportedCEMIMessage(
-                f"APCI not supported: {(tpci_apci & 0x03FF):#012b}"
+            self.tpci = TPCI.resolve(
+                raw_tpci=cemi[9 + addil], dst_is_group_address=dst_is_group_address
             )
+        except ConversionError as err:
+            raise UnsupportedCEMIMessage(
+                f"TPCI not supported: {cemi[9 + addil]:#10b}"
+            ) from err
+
+        if self.tpci.control:
+            if npdu_len:
+                raise UnsupportedCEMIMessage(
+                    f"Invalid length for control TPDU {self.tpci}: {npdu_len}"
+                )
+            return 10 + addil
+
+        _apci = apdu[0] * 256 + apdu[1]
+        try:
+            self.payload = APCI.resolve_apci(_apci)
+        except ConversionError as err:
+            raise UnsupportedCEMIMessage(f"APCI not supported: {_apci:#012b}") from err
 
         self.payload.from_knx(apdu)
 
-        return 10 + addil + self.mpdu_len
+        return 10 + addil + npdu_len
 
     def to_knx(self) -> bytes:
         """Serialize to KNX/IP raw data."""
-        if not isinstance(self.payload, APCI):
-            raise TypeError()
-        if not isinstance(self.src_addr, (GroupAddress, IndividualAddress)):
-            raise ConversionError("src_addr not set")
+        if self.tpci.control:
+            tpdu = bytes([self.tpci.to_knx()])
+            npdu_len = 0
+        else:
+            if not isinstance(self.payload, APCI):
+                raise ConversionError(
+                    f"Invalid payload set for data TPDU: {self.payload.__class__}"
+                )
+            tpdu = self.payload.to_knx()
+            tpdu[0] |= self.tpci.to_knx()
+            npdu_len = self.payload.calculated_length()
+
+        if not isinstance(self.src_addr, IndividualAddress):
+            raise ConversionError("src_addr invalid")
         if not isinstance(self.dst_addr, (GroupAddress, IndividualAddress)):
-            raise ConversionError("dst_addr not set")
+            raise ConversionError("dst_addr invalid")
 
         return (
             bytes(
@@ -188,20 +223,21 @@ class CEMIFrame:
                 (
                     *self.src_addr.to_knx(),
                     *self.dst_addr.to_knx(),
-                    self.payload.calculated_length(),
+                    npdu_len,
                 )
             )
-            + self.payload.to_knx()
+            + tpdu
         )
 
     def __repr__(self) -> str:
         """Return object as readable string."""
         return (
             "<CEMIFrame "
-            f'SourceAddress="{self.src_addr.__repr__()}" '
-            f'DestinationAddress="{self.dst_addr.__repr__()}" '
-            f'Flags="{self.flags:16b}" '
             f'code="{self.code.name}" '
+            f'src_addr="{self.src_addr.__repr__()}" '
+            f'dst_addr="{self.dst_addr.__repr__()}" '
+            f'flags="{self.flags:16b}" '
+            f'tpci="{self.tpci}" '
             f'payload="{self.payload}" />'
         )
 
